# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) Bolloré telecom, Polyconseil
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Granadilla

@author: Aymeric Augustin <aymeric.augustin@bolloretelecom.eu>
@author: Jeremy Lainé <jeremy.laine@bolloretelecom.eu>
@author: Raphaël Barrois <raphael.barrois@polyconseil.fr
"""

__docformat__ = 'epytext en'

__url__ = 'https://github.com/Polyconseil/granadilla'
__copyright__ = "(C) Bolloré Telecom, Polyconseil"
__license__ = 'GPL'
__license_long__ = """
 Copyright (C) Bolloré Telecom, Polyconseil
  All rights reserved."""
